@@ -12,6 +12,7 @@ import logging
 import threading
 import os
 import io
+import time
 from . import pipeline
 from azure.iot.device.common.auth import connection_string as cs
 from azure.iot.device.common.auth import sastoken as st
@@ -67,6 +68,24 @@ def _form_sas_uri(hostname, device_id, module_id=None):
         return "{hostname}/devices/{device_id}".format(hostname=hostname, device_id=device_id)
 
 
+def _extract_sas_uri_values(uri):
+    d = {}
+    items = uri.split("/")
+    if len(items) != 3 and len(items) != 5:
+        raise ValueError("Invalid SAS URI")
+    if items[1] != "devices":
+        raise ValueError("Cannot extract device id from SAS URI")
+    if len(items) > 3 and items[3] != "modules":
+        raise ValueError("Cannot extract module id from SAS URI")
+    d["hostname"] = items[0]
+    d["device_id"] = items[2]
+    try:
+        d["module_id"] = items[4]
+    except IndexError:
+        d["module_id"] = None
+    return d
+
+
 # Receive Type constant defs
 RECEIVE_TYPE_NONE_SET = "none_set"  # Type of receiving has not been set
 RECEIVE_TYPE_HANDLER = "handler"  # Only use handlers for receive
@@ -135,6 +154,45 @@ class AbstractIoTHubClient(object):
             else:
                 pass
 
+    def _replace_user_supplied_sastoken(self, sastoken_str):
+        """
+        Replaces the pipeline's NonRenewableSasToken with a new one based on a provided
+        sastoken string. Also does validation.
+        This helper only updates the PipelineConfig - it does not reauthorize the connection.
+        """
+        if not isinstance(
+            self._mqtt_pipeline.pipeline_configuration.sastoken, st.NonRenewableSasToken
+        ):
+            raise exceptions.ClientError(
+                "Cannot update sastoken when client was not created with one"
+            )
+        # Create new SasToken
+        try:
+            new_token_o = st.NonRenewableSasToken(sastoken_str)
+        except st.SasTokenError as e:
+            new_err = ValueError("Invalid SasToken provided")
+            new_err.__cause__ = e
+            raise new_err
+        # Extract values from SasToken
+        vals = _extract_sas_uri_values(new_token_o.resource_uri)
+        # Validate new token
+        if type(self).__name__ == "IoTHubDeviceClient" and vals["module_id"]:
+            raise ValueError("Provided SasToken is for a module")
+        if type(self).__name__ == "IoTHubModuleClient" and not vals["module_id"]:
+            raise ValueError("Provided SasToken is for a device")
+        if self._mqtt_pipeline.pipeline_configuration.device_id != vals["device_id"]:
+            raise ValueError("Provided SasToken does not match existing device id")
+        if self._mqtt_pipeline.pipeline_configuration.module_id != vals["module_id"]:
+            raise ValueError("Provided SasToken does not match existing module id")
+        if self._mqtt_pipeline.pipeline_configuration.hostname != vals["hostname"]:
+            raise ValueError("Provided SasToken does not match existing hostname")
+        if new_token_o.expiry_time < int(time.time()):
+            raise ValueError("Provided SasToken has already expired")
+        # Set token
+        # NOTE: We only need to set this on MQTT because this is a reference to the same object
+        # that is stored in HTTP. The HTTP pipeline is updated implicitly.
+        self._mqtt_pipeline.pipeline_configuration.sastoken = new_token_o
+
     @classmethod
     def create_from_connection_string(cls, connection_string, **kwargs):
         """
@@ -156,10 +214,10 @@ class AbstractIoTHubClient(object):
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
         :param int sastoken_ttl: The time to live (in seconds) for the created SasToken used for
             authentication. Default is 3600 seconds (1 hour)
-        :param int keepalive: Maximum period in seconds between communications with the
-        broker. If no other messages are being exchanged, this controls the
-        rate at which the client will send ping messages to the broker.
-        If not provided default value of 60 secs will be used.
+        :param int keep_alive: Maximum period in seconds between communications with the
+            broker. If no other messages are being exchanged, this controls the
+            rate at which the client will send ping messages to the broker.
+            If not provided default value of 60 secs will be used.
 
         :raises: ValueError if given an invalid connection_string.
         :raises: TypeError if given an unsupported parameter.
@@ -183,7 +241,7 @@ class AbstractIoTHubClient(object):
         )
         token_ttl = kwargs.get("sastoken_ttl", 3600)
         try:
-            sastoken = st.SasToken(uri, signing_mechanism, ttl=token_ttl)
+            sastoken = st.RenewableSasToken(uri, signing_mechanism, ttl=token_ttl)
         except st.SasTokenError as e:
             new_err = ValueError("Could not create a SasToken using provided values")
             new_err.__cause__ = e
@@ -207,6 +265,69 @@ class AbstractIoTHubClient(object):
 
         return cls(mqtt_pipeline, http_pipeline)
 
+    @classmethod
+    def create_from_sastoken(cls, sastoken, **kwargs):
+        """Instantiate the client from a pre-created SAS Token string
+
+        :param str sastoken: The SAS Token string
+
+        :param str server_verification_cert: Configuration Option. The trusted certificate chain.
+            Necessary when using connecting to an endpoint which has a non-standard root of trust,
+            such as a protocol gateway.
+        :param bool websockets: Configuration Option. Default is False. Set to true if using MQTT
+            over websockets.
+        :param cipher: Configuration Option. Cipher suite(s) for TLS/SSL, as a string in
+            "OpenSSL cipher list format" or as a list of cipher suite strings.
+        :type cipher: str or list(str)
+        :param str product_info: Configuration Option. Default is empty string. The string contains
+            arbitrary product info which is appended to the user agent string.
+        :param proxy_options: Options for sending traffic through proxy servers.
+        :type proxy_options: :class:`azure.iot.device.ProxyOptions`
+        :param int keep_alive: Maximum period in seconds between communications with the
+            broker. If no other messages are being exchanged, this controls the
+            rate at which the client will send ping messages to the broker.
+            If not provided default value of 60 secs will be used.
+
+        :raises: TypeError if given an unsupported parameter.
+        :raises: ValueError if the sastoken parameter is invalid.
+        """
+        # Ensure no invalid kwargs were passed by the user
+        excluded_kwargs = ["sastoken_ttl"]
+        _validate_kwargs(exclude=excluded_kwargs, **kwargs)
+
+        # Create SasToken object from string
+        try:
+            sastoken_o = st.NonRenewableSasToken(sastoken)
+        except st.SasTokenError as e:
+            new_err = ValueError("Invalid SasToken provided")
+            new_err.__cause__ = e
+            raise new_err
+        # Extract values from SasToken
+        vals = _extract_sas_uri_values(sastoken_o.resource_uri)
+        if cls.__name__ == "IoTHubDeviceClient" and vals["module_id"]:
+            raise ValueError("Provided SasToken is for a module")
+        if cls.__name__ == "IoTHubModuleClient" and not vals["module_id"]:
+            raise ValueError("Provided SasToken is for a device")
+        if sastoken_o.expiry_time < int(time.time()):
+            raise ValueError("Provided SasToken has already expired")
+        # Pipeline Config setup
+        config_kwargs = _get_config_kwargs(**kwargs)
+        pipeline_configuration = pipeline.IoTHubPipelineConfig(
+            device_id=vals["device_id"],
+            module_id=vals["module_id"],
+            hostname=vals["hostname"],
+            sastoken=sastoken_o,
+            **config_kwargs
+        )
+        if cls.__name__ == "IoTHubDeviceClient":
+            pipeline_configuration.blob_upload = True  # Blob Upload is a feature on Device Clients
+
+        # Pipeline setup
+        http_pipeline = pipeline.HTTPPipeline(pipeline_configuration)
+        mqtt_pipeline = pipeline.MQTTPipeline(pipeline_configuration)
+
+        return cls(mqtt_pipeline, http_pipeline)
+
     @abc.abstractmethod
     def connect(self):
         pass
@@ -215,6 +336,10 @@ class AbstractIoTHubClient(object):
     def disconnect(self):
         pass
 
+    @abc.abstractmethod
+    def update_sastoken(self, sastoken):
+        pass
+
     @abc.abstractmethod
     def send_message(self, message):
         pass
@@ -264,7 +389,7 @@ class AbstractIoTHubDeviceClient(AbstractIoTHubClient):
     @classmethod
     def create_from_x509_certificate(cls, x509, hostname, device_id, **kwargs):
         """
-        Instantiate a client which using X509 certificate authentication.
+        Instantiate a client using X509 certificate authentication.
 
         :param str hostname: Host running the IotHub.
             Can be found in the Azure portal in the Overview tab as the string hostname.
@@ -287,10 +412,10 @@ class AbstractIoTHubDeviceClient(AbstractIoTHubClient):
             arbitrary product info which is appended to the user agent string.
         :param proxy_options: Options for sending traffic through proxy servers.
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
-        :param int keepalive: Maximum period in seconds between communications with the
-        broker. If no other messages are being exchanged, this controls the
-        rate at which the client will send ping messages to the broker.
-        If not provided default value of 60 secs will be used.
+        :param int keep_alive: Maximum period in seconds between communications with the
+            broker. If no other messages are being exchanged, this controls the
+            rate at which the client will send ping messages to the broker.
+            If not provided default value of 60 secs will be used.
 
         :raises: TypeError if given an unsupported parameter.
 
@@ -337,12 +462,13 @@ class AbstractIoTHubDeviceClient(AbstractIoTHubClient):
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
         :param int sastoken_ttl: The time to live (in seconds) for the created SasToken used for
             authentication. Default is 3600 seconds (1 hour)
-        :param int keepalive: Maximum period in seconds between communications with the
-        broker. If no other messages are being exchanged, this controls the
-        rate at which the client will send ping messages to the broker.
-        If not provided default value of 60 secs will be used.
+        :param int keep_alive: Maximum period in seconds between communications with the
+            broker. If no other messages are being exchanged, this controls the
+            rate at which the client will send ping messages to the broker.
+            If not provided default value of 60 secs will be used.
 
         :raises: TypeError if given an unsupported parameter.
+        :raises: ValueError if the provided parameters are invalid.
 
         :return: An instance of an IoTHub client that uses a symmetric key for authentication.
         """
@@ -354,7 +480,7 @@ class AbstractIoTHubDeviceClient(AbstractIoTHubClient):
         signing_mechanism = auth.SymmetricKeySigningMechanism(key=symmetric_key)
         token_ttl = kwargs.get("sastoken_ttl", 3600)
         try:
-            sastoken = st.SasToken(uri, signing_mechanism, ttl=token_ttl)
+            sastoken = st.RenewableSasToken(uri, signing_mechanism, ttl=token_ttl)
         except st.SasTokenError as e:
             new_err = ValueError("Could not create a SasToken using provided values")
             new_err.__cause__ = e
@@ -409,10 +535,10 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
         :param int sastoken_ttl: The time to live (in seconds) for the created SasToken used for
             authentication. Default is 3600 seconds (1 hour)
-        :param int keepalive: Maximum period in seconds between communications with the
-        broker. If no other messages are being exchanged, this controls the
-        rate at which the client will send ping messages to the broker.
-        If not provided default value of 60 secs will be used.
+        :param int keep_alive: Maximum period in seconds between communications with the
+            broker. If no other messages are being exchanged, this controls the
+            rate at which the client will send ping messages to the broker.
+            If not provided default value of 60 secs will be used.
 
         :raises: OSError if the IoT Edge container is not configured correctly.
         :raises: ValueError if debug variables are invalid.
@@ -498,7 +624,7 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
         uri = _form_sas_uri(hostname=hostname, device_id=device_id, module_id=module_id)
         token_ttl = kwargs.get("sastoken_ttl", 3600)
         try:
-            sastoken = st.SasToken(uri, signing_mechanism, ttl=token_ttl)
+            sastoken = st.RenewableSasToken(uri, signing_mechanism, ttl=token_ttl)
         except st.SasTokenError as e:
             new_err = ValueError(
                 "Could not create a SasToken using the values provided, or in the Edge environment"
@@ -530,7 +656,7 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
     @classmethod
     def create_from_x509_certificate(cls, x509, hostname, device_id, module_id, **kwargs):
         """
-        Instantiate a client which using X509 certificate authentication.
+        Instantiate a client using X509 certificate authentication.
 
         :param str hostname: Host running the IotHub.
             Can be found in the Azure portal in the Overview tab as the string hostname.
@@ -554,10 +680,10 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
             arbitrary product info which is appended to the user agent string.
         :param proxy_options: Options for sending traffic through proxy servers.
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
-        :param int keepalive: Maximum period in seconds between communications with the
-        broker. If no other messages are being exchanged, this controls the
-        rate at which the client will send ping messages to the broker.
-        If not provided default value of 60 secs will be used.
+        :param int keep_alive: Maximum period in seconds between communications with the
+            broker. If no other messages are being exchanged, this controls the
+            rate at which the client will send ping messages to the broker.
+            If not provided default value of 60 secs will be used.
 
         :raises: TypeError if given an unsupported parameter.
 
